@@ -1,3 +1,4 @@
+from typing import List
 from urllib.parse import urljoin
 
 from django.conf import settings
@@ -13,7 +14,7 @@ from guardian.shortcuts import get_users_with_perms
 from ephios.core.models import AbstractParticipation, Event, LocalParticipation, UserProfile
 from ephios.core.models.users import Consequence, Notification
 from ephios.core.signals import register_notification_types
-from ephios.core.signup import LocalUserParticipant
+from ephios.core.signup.participants import LocalUserParticipant
 
 
 def installed_notification_types():
@@ -83,7 +84,7 @@ class ProfileUpdateNotification(AbstractNotificationHandler):
 
     @classmethod
     def get_url(cls, notification):
-        return urljoin(settings.SITE_URL, reverse("core:profile"))
+        return urljoin(settings.GET_SITE_URL(), reverse("core:profile"))
 
 
 class NewProfileNotification(AbstractNotificationHandler):
@@ -110,14 +111,14 @@ class NewProfileNotification(AbstractNotificationHandler):
             "You're receiving this email because a new account has been created for you at ephios.\n"
             "Please go to the following page and choose a password: {url}\n"
             "Your username is your email address: {email}\n"
-        ).format(url=urljoin(settings.SITE_URL, reset_link), email=notification.user.email)
+        ).format(url=urljoin(settings.GET_SITE_URL(), reset_link), email=notification.user.email)
 
     @classmethod
     def as_html(cls, notification):
         return render_to_string(
             "core/mails/new_account_email.html",
             {
-                **{"site_url": settings.SITE_URL, "email": notification.user.email},
+                **{"site_url": settings.GET_SITE_URL(), "email": notification.user.email},
                 **notification.data,
             },
         )
@@ -160,18 +161,22 @@ class NewEventNotification(AbstractNotificationHandler):
     def as_html(cls, notification):
         event = Event.objects.get(pk=notification.data.get("event_id"))
         return render_to_string(
-            "core/mails/new_event.html", {"event": event, "site_url": settings.SITE_URL}
+            "core/mails/new_event.html", {"event": event, "site_url": settings.GET_SITE_URL()}
         )
 
     @classmethod
     def get_url(cls, notification):
         event = Event.objects.get(pk=notification.data.get("event_id"))
-        return urljoin(settings.SITE_URL, event.get_absolute_url())
+        return urljoin(settings.GET_SITE_URL(), event.get_absolute_url())
 
 
-class ParticipationConfirmedNotification(AbstractNotificationHandler):
-    slug = "ephios_participation_confirmed"
-    title = _("Your participation has been confirmed")
+class ParticipationMixin:
+    @classmethod
+    def get_url(cls, notification):
+        shift = AbstractParticipation.objects.get(
+            id=notification.data.get("participation_id")
+        ).shift
+        return urljoin(settings.GET_SITE_URL(), shift.get_absolute_url())
 
     @classmethod
     def send(cls, participation: AbstractParticipation):
@@ -186,6 +191,11 @@ class ParticipationConfirmedNotification(AbstractNotificationHandler):
             data=dict(participation_id=participation.id, email=participation.participant.email),
         )
 
+
+class ParticipationConfirmedNotification(ParticipationMixin, AbstractNotificationHandler):
+    slug = "ephios_participation_confirmed"
+    title = _("Your participation has been confirmed")
+
     @classmethod
     def get_subject(cls, notification):
         event = AbstractParticipation.objects.get(
@@ -195,36 +205,21 @@ class ParticipationConfirmedNotification(AbstractNotificationHandler):
 
     @classmethod
     def as_plaintext(cls, notification):
-        shift = AbstractParticipation.objects.get(
+        participation: AbstractParticipation = AbstractParticipation.objects.get(
             id=notification.data.get("participation_id")
-        ).shift
-        return _("Your participation for {shift} is now confirmed.").format(shift=shift)
-
-    @classmethod
-    def get_url(cls, notification):
-        event = AbstractParticipation.objects.get(
-            id=notification.data.get("participation_id")
-        ).shift.event
-        return urljoin(settings.SITE_URL, event.get_absolute_url())
+        )
+        message = _("Your participation for {shift} is now confirmed.").format(
+            shift=participation.shift
+        )
+        if participation.has_customized_signup():
+            message += f'\n\n{_("Your time is")} {participation.get_time_display()}'
+        return message
 
 
-class ParticipationRejectedNotification(AbstractNotificationHandler):
+class ParticipationRejectedNotification(ParticipationMixin, AbstractNotificationHandler):
     slug = "ephios_participation_rejected"
     title = _("Your participation has been rejected")
 
-    @classmethod
-    def send(cls, participation: AbstractParticipation):
-        user = (
-            participation.user
-            if participation.get_real_instance_class() == LocalParticipation
-            else None
-        )
-        Notification.objects.create(
-            slug=cls.slug,
-            user=user,
-            data=dict(participation_id=participation.id, email=participation.participant.email),
-        )
-
     @classmethod
     def get_subject(cls, notification):
         event = AbstractParticipation.objects.get(
@@ -241,44 +236,87 @@ class ParticipationRejectedNotification(AbstractNotificationHandler):
             shift=shift
         )
 
+
+class ParticipationCustomizationNotification(ParticipationMixin, AbstractNotificationHandler):
+    slug = "ephios_participation_customized"
+    title = _("A confirmed participation of yours has been tweaked by a responsible")
+
+    @classmethod
+    def send(
+        cls, participation: AbstractParticipation, claims: List[str]
+    ):  # pylint: disable=arguments-differ
+        user = (
+            participation.user
+            if participation.get_real_instance_class() == LocalParticipation
+            else None
+        )
+        Notification.objects.create(
+            slug=cls.slug,
+            user=user,
+            data=dict(
+                participation_id=participation.id,
+                email=participation.participant.email,
+                claims=claims,
+            ),
+        )
+
+    @classmethod
+    def get_subject(cls, notification):
+        shift = AbstractParticipation.objects.get(
+            id=notification.data.get("participation_id")
+        ).shift
+        return _("Participation tweaked for {shift}").format(shift=shift)
+
+    @classmethod
+    def as_plaintext(cls, notification):
+        shift = AbstractParticipation.objects.get(
+            id=notification.data.get("participation_id")
+        ).shift
+        message = _(
+            "Your participation for {shift} has been tweaked by a responsible user."
+        ).format(shift=shift)
+        message += "\n\n" + "\n".join(f"- {claim}" for claim in notification.data.get("claims", []))
+        return message
+
+
+class ResponsibleMixin:
+    def __init__(self, participation: AbstractParticipation):
+        self.participation = participation
+
+    def responsible_users(self):
+        users = get_users_with_perms(
+            self.participation.shift.event, only_with_perms_in=["change_event"]
+        ).distinct()
+        for user in users:
+            if (
+                self.participation.get_real_instance_class() != LocalParticipation
+                or user != self.participation.user
+            ):
+                yield user
+
+    def get_data(self, **kwargs):
+        kwargs["disposition_url"] = urljoin(
+            settings.GET_SITE_URL(),
+            reverse("core:shift_disposition", kwargs=dict(pk=self.participation.shift.pk)),
+        )
+        kwargs["participation_id"] = self.participation.id
+        return kwargs
+
+    def send(self):
+        Notification.objects.bulk_create(
+            [
+                Notification(
+                    slug=self.slug,
+                    user=user,
+                    data=self.get_data(),
+                )
+                for user in self.responsible_users()
+            ]
+        )
+
     @classmethod
     def get_url(cls, notification):
-        event = AbstractParticipation.objects.get(
-            id=notification.data.get("participation_id")
-        ).shift.event
-        return urljoin(settings.SITE_URL, event.get_absolute_url())
-
-
-class ResponsibleParticipationRequested(AbstractNotificationHandler):
-    slug = "ephios_participation_responsible_requested"
-    title = _("A participation has been requested for your event")
-
-    @classmethod
-    def send(cls, participation: AbstractParticipation):
-        responsible_users = get_users_with_perms(
-            participation.shift.event, only_with_perms_in=["change_event"]
-        ).distinct()
-        disposition_url = urljoin(
-            settings.SITE_URL,
-            reverse("core:shift_disposition", kwargs=dict(pk=participation.shift.pk)),
-        )
-        notifications = []
-        for user in responsible_users:
-            if (
-                not participation.get_real_instance_class() == LocalParticipation
-                or user != participation.user
-            ):
-                notifications.append(
-                    Notification(
-                        slug=cls.slug,
-                        user=user,
-                        data=dict(
-                            participation_id=participation.id,
-                            disposition_url=disposition_url,
-                        ),
-                    )
-                )
-        Notification.objects.bulk_create(notifications)
+        return notification.data.get("disposition_url")
 
     @classmethod
     def get_subject(cls, notification):
@@ -290,6 +328,11 @@ class ResponsibleParticipationRequested(AbstractNotificationHandler):
             state=participation_state, event=participation.shift.event
         )
 
+
+class ResponsibleParticipationRequestedNotification(ResponsibleMixin, AbstractNotificationHandler):
+    slug = "ephios_participation_responsible_requested"
+    title = _("A participation has been requested for your event")
+
     @classmethod
     def as_plaintext(cls, notification):
         participation = AbstractParticipation.objects.get(
@@ -307,9 +350,53 @@ class ResponsibleParticipationRequested(AbstractNotificationHandler):
             participant=participation.participant, shift=participation.shift
         )
 
+
+class ResponsibleConfirmedParticipationDeclinedNotification(
+    ResponsibleMixin, AbstractNotificationHandler
+):
+    slug = "ephios_participation_responsible_confirmed_declined"
+    title = _("A participant declined after having been confirmed for your event")
+
     @classmethod
-    def get_url(cls, notification):
-        return notification.data.get("disposition_url")
+    def as_plaintext(cls, notification):
+        participation = AbstractParticipation.objects.get(
+            id=notification.data.get("participation_id")
+        )
+        return _("{participant} declined their participation in {shift}.").format(
+            participant=participation.participant, shift=participation.shift
+        )
+
+
+class ResponsibleConfirmedParticipationCustomizedNotification(
+    ResponsibleMixin, AbstractNotificationHandler
+):
+    slug = "ephios_participation_responsible_confirmed_customized"
+    title = _("A confirmed participant altered their participation")
+
+    @classmethod
+    def get_subject(cls, notification):
+        participation = AbstractParticipation.objects.get(
+            id=notification.data.get("participation_id")
+        )
+        return _("Participation altered for {event}").format(event=participation.shift.event)
+
+    def __init__(self, participation: AbstractParticipation, claims: List[str]):
+        super().__init__(participation)
+        self.claims = claims
+
+    def get_data(self, **kwargs):
+        return super().get_data(claims=self.claims)
+
+    @classmethod
+    def as_plaintext(cls, notification):
+        participation = AbstractParticipation.objects.get(
+            id=notification.data.get("participation_id")
+        )
+        message = _("{participant} altered their participation in {shift}.").format(
+            participant=participation.participant, shift=participation.shift
+        )
+        message += "\n\n" + "\n".join(f"- {claim}" for claim in notification.data.get("claims", []))
+        return message
 
 
 class EventReminderNotification(AbstractNotificationHandler):
@@ -351,7 +438,7 @@ class EventReminderNotification(AbstractNotificationHandler):
     @classmethod
     def get_url(cls, notification):
         event = Event.objects.get(pk=notification.data.get("event_id"))
-        return urljoin(settings.SITE_URL, event.get_absolute_url())
+        return urljoin(settings.GET_SITE_URL(), event.get_absolute_url())
 
 
 class CustomEventParticipantNotification(AbstractNotificationHandler):
@@ -437,7 +524,10 @@ CORE_NOTIFICATION_TYPES = [
     NewProfileNotification,
     ParticipationRejectedNotification,
     ParticipationConfirmedNotification,
-    ResponsibleParticipationRequested,
+    ParticipationCustomizationNotification,
+    ResponsibleParticipationRequestedNotification,
+    ResponsibleConfirmedParticipationDeclinedNotification,
+    ResponsibleConfirmedParticipationCustomizedNotification,
     NewEventNotification,
     EventReminderNotification,
     CustomEventParticipantNotification,
