@@ -6,7 +6,7 @@ from guardian.shortcuts import get_objects_for_user
 
 from ephios.core.consequences import editable_consequences, pending_consequences
 from ephios.core.models import AbstractParticipation, Shift
-from ephios.core.signup import get_conflicting_participations
+from ephios.core.signup.methods import get_conflicting_participations
 
 register = template.Library()
 
@@ -46,11 +46,14 @@ def get_relevant_qualifications(qualification_queryset):
     return qs.values_list("abbreviation", flat=True)
 
 
-@register.filter(name="conflicting_shifts")
-def participant_conflicting_shifts(participant, shift):
-    return get_conflicting_participations(shift, participant).values_list(
-        "shift__event__title", flat=True
-    )
+@register.filter(name="conflicting_participations")
+def participation_conflicts(participation):
+    return get_conflicting_participations(
+        participant=participation.participant,
+        shift=participation.shift,
+        start_time=participation.start_time,
+        end_time=participation.end_time,
+    ).values_list("shift__event__title", flat=True)
 
 
 @register.filter(name="shifts_needing_disposition")
